#include <stdio.h>
#include <ctype.h>
#include <proto.h>

#include "common.h"
#include "error.h"
#include "dlink.h"

/************************************************************************\
*
* Tag management module for 'tagorg'.
*
* $Copyright: (C) 1997 by Gran Gustafsson $ 
*
\************************************************************************/

/************************************************************************\
*
* Add tag to list. Return true if added.
*
\************************************************************************/
sbool AddTag(char *name,DLink *lhd)
{
	TList *tl;
	
	for(tl=(TList *)dl_First(lhd); tl; tl=(TList *)dl_Next(lhd,&tl->lnk))
	{
		if(strcmp(tl->name,name)==0)
			break;
	}
	if(tl)
		return(FALSE);
	if(!(tl=(TList *)malloc(sizeof(TList)+strlen(name))))
		Error(ERR_FATAL,"No memory for adding tag");
	strcpy(tl->name,name);
	dl_AddTail(lhd,&tl->lnk);
	return(TRUE);
}

/************************************************************************\
*
* Remove tag from list.
*
\************************************************************************/
sbool RemTag(char *name,DLink *lhd)
{
	TList *tl;
	
	for(tl=(TList *)dl_First(lhd); tl; tl=(TList *)dl_Next(lhd,&tl->lnk))
	{
		if(strcmp(tl->name,name)==0)
			break;
	}
	if(!tl)
		return(FALSE);
	dl_Remove(&tl->lnk);
	free(tl);
	return(TRUE);
}

/************************************************************************\
*
* Check if tag is in list.
*
\************************************************************************/
sbool FindTag(char *name,DLink *lhd)
{
	TList *tl;

	for(tl=(TList *)dl_First(lhd); tl; tl=(TList *)dl_Next(lhd,&tl->lnk))
	{
		if(strcmp(tl->name,name)==0)
			break;
	}
	if(!tl)
		return(FALSE);
	return(TRUE);
}

/************************************************************************\
*
* Generate header file from taglist.
*
\************************************************************************/
void Generate(char *name,char *pref,char *start,int nr,char *inc,DLink *lhd)
{
	FILE *out;
	TList *tl;
	char buf[500];
	int n;
	
	if((out=fopen(name,"w")))
	{
		for(n=0; name[n] && (n<499) && (isalnum(name[n])); n++)
			buf[n]=toupper(name[n]);
		buf[n]=0;
		fprintf(out,"#ifndef\tINC_%s_H\n#define\tINC_%s_H\n\n",buf,buf);
		fprintf(out,"/* Header generated by TagOrganizer */\n\n");
		fprintf(out,"#include <%s>\n\n",inc);
		fprintf(out,"#define %sBASETAG%d\t(%s)\n\n",pref,nr,start);
		n=1;
		for(tl=(TList *)dl_First(lhd); tl; tl=(TList *)dl_Next(lhd,&tl->lnk))
		{
			fprintf(out,"#define %s%s\t(%sBASETAG%d+%d)\n",pref,tl->name,pref,nr,n);
			n++;
		}
		fprintf(out,"\n#define %sLAST_TAG%d\t(%sBASETAG%d+%d)\n\n",pref,nr,pref,nr,n);
		fprintf(out,"#endif /* INC_%s_H */\n",buf);
		fclose(out);
	}
	else
		Error(ERR_FATAL,"Unable to write headerfile");
}

/************************************************************************\
*
* Print tags in list.
*
\************************************************************************/
void ListTags(DLink *lhd)
{
	TList *tl;

	printf("  ** Contents of tagfile **\n\n");	
	for(tl=(TList *)dl_First(lhd); tl; tl=(TList *)dl_Next(lhd,&tl->lnk))
	{
		printf("%s\n",tl->name);
	}
}	
